# -*- coding: utf-8 -*-
# Generated by Django 1.9.1 on 2016-09-12 09:23
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [("contentstore", "0004_auto_20160810_0916")]

    operations = [
        migrations.AlterUniqueTogether(
            name="message",
            unique_together=set([("messageset", "sequence_number", "lang")]),
        )
    ]
