# -*- coding: utf-8 -*-
# Generated by Django 1.11.13 on 2018-10-18 07:29
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [("contentstore", "0008_schedule_scheduler_schedule_id")]

    operations = [
        migrations.AddField(
            model_name="messageset",
            name="label",
            field=models.CharField(
                blank=True,
                default=b"",
                max_length=100,
                verbose_name="User-readable name",
            ),
        )
    ]
