{% extends "admin/change_list.html" %} {% load i18n %}
{% block object-tools-items %}
    {{ block.super }}
    {% if perms.subscriptions.can_find_behind_subscriptions %}
        <li>
            <a class="historylink" href={% url "admin:find_behind_subscriptions" %}>
                Find behind subscriptions
            </a>
        </li>
    {% endif %}
{% endblock %}
