# -*- coding: utf-8 -*-
# Generated by Django 1.9.12 on 2017-02-13 09:59
from __future__ import unicode_literals

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [("subscriptions", "0004_subscription_initial_sequence_number")]

    operations = [
        migrations.CreateModel(
            name="SubscriptionSendFailure",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("task_id", models.UUIDField()),
                ("initiated_at", models.DateTimeField()),
                ("reason", models.TextField()),
                (
                    "subscription",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="subscriptions.Subscription",
                    ),
                ),
            ],
        )
    ]
